/*
 * This file is part of Linode Manager Classic.
 *
 * Linode Manager Classic is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Linode Manager Classic is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Linode Manager Classic.  If not, see <https://www.gnu.org/licenses/>.
 */

import { settings, elements, apiDelete, apiGet, parseParams, setupHeader } from "/global.js";

(function()
{
	// Element names specific to this page
	elements.deleteButton = "delete-button";
	elements.domain = "domain";

	// Data recieved from API calls
	var data = {};

	// Static references to UI elements
	var ui = {};
	ui.deleteButton = {};
	ui.domain = {};

	// Callback for domain details API call
	var displayDetails = function(response)
	{
		// Set page title and header stuff
		document.title += " // " + response.domain;
		ui.domain.innerHTML = response.domain;
		ui.deleteButton.disabled = false;
	};

	// Click handler for delete button
	var handleDelete = function(event)
	{
		if (event.currentTarget.disabled)
			return;

		apiDelete("/domains/" + data.params.did, function(response)
		{
			location.href = "/dns";
		});
	};

	// Initial setup
	var setup = function()
	{
		// Parse URL parameters
		data.params = parseParams();

		// We need a domain ID, so die if we don't have it
		if (!data.params.did) {
			alert("No domain ID supplied!");
			return;
		}

		setupHeader();

		// Update links on page to include proper Linode ID
		var anchors = document.getElementsByTagName("a");
		for (var i = 0; i < anchors.length; i++)
			anchors[i].href = anchors[i].href.replace("did=0", "did=" + data.params.did);

		// Get element references
		ui.deleteButton = document.getElementById(elements.deleteButton);
		ui.domain = document.getElementById(elements.domain);

		// Attach event handlers
		ui.deleteButton.addEventListener("click", handleDelete);

		// Get data from API
		apiGet("/domains/" + data.params.did, displayDetails, null);
	};

	// Attach onload handler
	window.addEventListener("load", setup);
})();
