/*
 * This file is part of Linode Manager Classic.
 *
 * Linode Manager Classic is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Linode Manager Classic is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Linode Manager Classic.  If not, see <https://www.gnu.org/licenses/>.
 */

import { settings, elements, apiGet, parseParams, setupHeader } from "/global.js";

(function()
{
	// Element names specific to this page
	elements.domainLabel = "domain-label";
	elements.domainTag = "domain-tag";
	elements.domainTagLink = "domain-tag-link";
	elements.zoneFile = "zonefile";

	// Data recieved from API calls
	var data = {};
	data.domain = {};

	// Static references to UI elements
	var ui = {};
	ui.domainLabel = {};
	ui.domainTag = {};
	ui.domainTagLink = {};
	ui.zoneFile = {};

	// Callback for domain details API call
	var displayDetails = function(response)
	{
		data.domain = response;

		// Set page title and header stuff
		document.title += " // " + data.domain.domain;
		ui.domainLabel.innerHTML = data.domain.domain;
		if (data.domain.tags.length == 1) {
			ui.domainTagLink.href = "/dns?tag=" + data.domain.tags[0];
			ui.domainTagLink.innerHTML = "(" + data.domain.tags[0] + ")";
			ui.domainTag.style.display = "inline";
		}
	};

	// Display zonefile contents
	var displayZoneFile = function(response)
	{
		ui.zoneFile.innerHTML = response.zone_file.join("\n");
	};

	// Initial setup
	var setup = function()
	{
		// Parse URL parameters
		data.params = parseParams();

		// We need a domain ID, so die if we don't have it
		if (!data.params.did) {
			alert("No domain ID supplied!");
			return;
		}

		setupHeader();

		// Update links on page to include proper domain ID
		var anchors = document.getElementsByTagName("a");
		for (var i = 0; i < anchors.length; i++)
			anchors[i].href = anchors[i].href.replace("did=0", "did=" + data.params.did);

		// Get element references
		ui.domainLabel = document.getElementById(elements.domainLabel);
		ui.domainTag = document.getElementById(elements.domainTag);
		ui.domainTagLink = document.getElementById(elements.domainTagLink);
		ui.zoneFile = document.getElementById(elements.zoneFile);

		// Get data from API
		apiGet("/domains/" + data.params.did, displayDetails, null);
		apiGet("/domains/" + data.params.did + "/zone-file", displayZoneFile, null);
	};

	// Attach onload handler
	window.addEventListener("load", setup);
})();
