/*
 * This file is part of Linode Manager Classic.
 *
 * Linode Manager Classic is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Linode Manager Classic is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Linode Manager Classic.  If not, see <https://www.gnu.org/licenses/>.
 */

import { settings, elements, apiPost, parseParams, setupHeader } from "/global.js";

(function()
{
	// Element names specific to this page
	elements.addButton = "add-button";
	elements.email = "email";
	elements.restrictedYes = "restricted-yes";
	elements.user = "user";

	// Data received from API calls
	var data = {};

	// Static references to UI elements
	var ui = {};
	ui.addButton = {};
	ui.email = {};
	ui.restrictedYes = {};
	ui.user = {};

	// Click handler for add button
	var handleAdd = function(event)
	{
		var req = {
			"username": ui.user.value,
			"email": ui.email.value,
			"restricted": ui.restrictedYes.checked
		};

		apiPost("/account/users", req, function(response)
		{
			location.href = "/user";
		});
	};

	// Initial setup
	var setup = function()
	{
		// Parse URL parameters
		data.params = parseParams();

		setupHeader();

		// Highlight the account nav link
		var navlinks = document.getElementsByClassName(elements.navlink);
		for (var i = 0; i < navlinks.length; i++) {
			if (navlinks[i].pathname == "/account/")
				navlinks[i].className = " " + elements.navlinkActive;
		}

		// Get element references
		ui.addButton = document.getElementById(elements.addButton);
		ui.email = document.getElementById(elements.email);
		ui.restrictedYes = document.getElementById(elements.restrictedYes);
		ui.user = document.getElementById(elements.user);

		// Attach event handlers
		ui.addButton.addEventListener("click", handleAdd);
	};

	// Attach onload handler
	window.addEventListener("load", setup);
})();
