/*
 * This file is part of Linode Manager Classic.
 *
 * Linode Manager Classic is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Linode Manager Classic is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Linode Manager Classic.  If not, see <https://www.gnu.org/licenses/>.
 */

import { settings, elements, apiGet, apiPut, md5, parseParams, setupHeader } from "/global.js";

(function()
{
	// Element names specific to this page
	elements.email = "email";
	elements.pwReset = "pw-reset";
	elements.restrictedNo = "restricted-no";
	elements.restrictedYes = "restricted-yes";
	elements.saveButton = "save-button";
	elements.topUserLabel = "top-user-label";
	elements.user = "user";
	elements.userLabel = "user-label";

	// Data received from API calls
	var data = {};

	// Static references to UI elements
	var ui = {};
	ui.email = {};
	ui.pwReset = {};
	ui.restrictedNo = {};
	ui.restrictedYes = {};
	ui.saveButton = {};
	ui.topUserLabel = {};
	ui.user = {};
	ui.userLabel = {};

	// Callback for user details API call
	var displayUser = function(response)
	{
		// Fill in form
		ui.user.value = response.username;
		ui.email.value = response.email;
		if (response.restricted)
			ui.restrictedYes.checked = true;
		else
			ui.restrictedNo.checked = true;
		ui.saveButton.disabled = false;
	};

	// Click handler for save button
	var handleSave = function(event)
	{
		if (event.currentTarget.disabled)
			return;

		var req = {
			"username": ui.user.value,
			"restricted": ui.restrictedYes.checked
		};

		apiPut("/account/users/" + data.params.user, req, function(response)
		{
			location.href = "/user";
		});
	};

	// Initial setup
	var setup = function()
	{
		// Parse URL parameters
		data.params = parseParams();

		// We need a username, so die if we don't have it
		if (!data.params.user) {
			alert("No username supplied!");
			return;
		}

		setupHeader();

		// Highlight the account nav link
		var navlinks = document.getElementsByClassName(elements.navlink);
		for (var i = 0; i < navlinks.length; i++) {
			if (navlinks[i].pathname == "/account/")
				navlinks[i].className = " " + elements.navlinkActive;
		}

		// Get element references
		ui.email = document.getElementById(elements.email);
		ui.pwReset = document.getElementById(elements.pwReset);
		ui.restrictedNo = document.getElementById(elements.restrictedNo);
		ui.restrictedYes = document.getElementById(elements.restrictedYes);
		ui.saveButton = document.getElementById(elements.saveButton);
		ui.topUserLabel = document.getElementById(elements.topUserLabel);
		ui.user = document.getElementById(elements.user);
		ui.userLabel = document.getElementById(elements.userLabel);

		// Populate username where we need it
		ui.topUserLabel.innerHTML = data.params.user;
		ui.userLabel.innerHTML = data.params.user;
		ui.pwReset.href += data.params.user;
		document.title += " - " + data.params.user;

		// Attach event handlers
		ui.saveButton.addEventListener("click", handleSave);

		// Get data from API
		apiGet("/account/users/" + data.params.user, displayUser, null);
	};

	// Attach onload handler
	window.addEventListener("load", setup);
})();
