/*
 * This file is part of Linode Manager Classic.
 *
 * Linode Manager Classic is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Linode Manager Classic is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Linode Manager Classic.  If not, see <https://www.gnu.org/licenses/>.
 */

import { settings, elements, apiDelete, apiGet, parseParams, setupHeader } from "/global.js";

(function()
{
	// Element names specific to this page
	elements.confirm = "confirm";
	elements.deleteButton = "delete-button";
	elements.label = "label";
	elements.volumeLabel = "volume-label";
	elements.warning = "warning";

	// Data recieved from API calls
	var data = {};

	// Static references to UI elements
	var ui = {};
	ui.confirm = {};
	ui.deleteButton = {};
	ui.label = {};
	ui.volumeLabel = {};
	ui.warning = {};

	// Callback for volume API call
	var displayVolume = function(response)
	{
		// Set page title and header stuff
		document.title += " // " + response.label;
		ui.volumeLabel.innerHTML = response.label;
		ui.label.innerHTML = response.label;
		ui.deleteButton.disabled = false;
	};

	// Click handler for delete button
	var handleDelete = function(event)
	{
		if (event.currentTarget.disabled)
			return;

		if (!ui.confirm.checked) {
			ui.warning.style.display = "block";
			return;
		}

		apiDelete("/volumes/" + data.params.vid, function(response)
		{
			location.href = "/volumes";
		});
	};

	// Initial setup
	var setup = function()
	{
		// Parse URL parameters
		data.params = parseParams();

		// We need a volume ID, so die if we don't have it
		if (!data.params.vid) {
			alert("No volume ID supplied!");
			return;
		}

		setupHeader();

		// Update links on page to include proper volume ID
		var anchors = document.getElementsByTagName("a");
		for (var i = 0; i < anchors.length; i++)
			anchors[i].href = anchors[i].href.replace("vid=0", "vid=" + data.params.vid);

		// Get element references
		ui.confirm = document.getElementById(elements.confirm);
		ui.deleteButton = document.getElementById(elements.deleteButton);
		ui.label = document.getElementById(elements.label);
		ui.volumeLabel = document.getElementById(elements.volumeLabel);
		ui.warning = document.getElementById(elements.warning);

		// Attach event handlers
		ui.deleteButton.addEventListener("click", handleDelete);

		// Get data from API
		apiGet("/volumes/" + data.params.vid, displayVolume, null);
	};

	// Attach onload handler
	window.addEventListener("load", setup);
})();
